#include <random>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <set>

#define MAXN 2001
#define MAXR 400000

std::random_device rd;
std::mt19937 gen(rd());
std::uniform_int_distribution<> dis1(3, 100);
std::uniform_int_distribution<> dis2(3, 10);
std::uniform_int_distribution<> dis3(3, 2000);
std::uniform_int_distribution<> disC(1, 3000);

// generate a graph for a given size n
void test_case(unsigned int n, unsigned int *costs, std::ofstream &out)
{
  unsigned int degree[MAXN];

  // distribution of node degrees
  std::uniform_int_distribution<> disG(2, n-1);

  // distribution of nodes
  std::uniform_int_distribution<> disN(1, n);

  // graph and node degrees/edge counters
  unsigned int e = 0;
  std::fill_n(degree, MAXN, 0);
  std::fill_n(&costs[0], MAXN*MAXN, 0);

  // generate out-edges for each node
  for(unsigned int u = 1; u <= n; u++)
  {
    // generate a minimum degree for u
    unsigned int du = disG(gen);

    // generate edges until minimum degree reached
    while (degree[u] < du)
    {
      // randomly choose destination node
      unsigned int v = disN(gen);

      // randomly choose cost
      unsigned int c = disC(gen);

      // insert new edge?
      if (!costs[u*MAXN+v] && !costs[v*MAXN+u] && u != v)
      {
        // store cost
        costs[u*MAXN+v] = c;

        // increment degree counters
        degree[u]++;
        degree[v]++;

        // increment edge counter
        e++;
      }
      if (e == MAXR) {
        u = n+1;
        break;
      }
    }
  }

  // number of nodes/edge
  out << n << " " << e << "\n";
  for(unsigned int u = 1; u <= n; u++)
    for(unsigned int v = 1; v <= n; v++)
      if (costs[u*MAXN+v])
        out << u << " " << v << " " << costs[u*MAXN+v] << "\n";
}

int make_one_file(const char* filename, std::uniform_int_distribution<> dd, int num_test_cases) {
  //////////////////////////////////////////////////////////////////////////////
  unsigned int *costs = new unsigned int[MAXN*MAXN];

  std::ofstream out(filename);
  out << num_test_cases << "\n";
  for(unsigned int i = 0; i < num_test_cases; i++)
  {
    std::cerr << ".";

    // number of nodes
    unsigned int n = dd(gen);

    // generate test case
    //if (i != 0)
      //out << "\n";

    test_case(n, costs, out);
  }

  out.close();
  std::cerr << "\n";


}

int main()
{
  srand (time(NULL));

  make_one_file("secret0.in", dis1, 10);
  make_one_file("secret1.in", dis1, 20);
  make_one_file("secret2.in", dis1, 30);
  make_one_file("secret3.in", dis1, 40);
  make_one_file("secret4.in", dis1, 50);
  make_one_file("secret5.in", dis1, 100);
  make_one_file("secret6.in", dis2, 5);
  make_one_file("secret7.in", dis2, 10);
  make_one_file("secret8.in", dis2, 50);
  make_one_file("secret9.in", dis3, 3);

  return 0;
}
